local mod = UpgradeList:NewModule("Import")
local L = LibStub("AceLocale-3.0"):GetLocale("UpgradeList")

local UPGRADELIST_VERSION = GetAddOnMetadata('UpgradeList', 'Version')

local RawrInputExplanation = "To import items marked as 'available' from a Rawr save file:\n\n  1) Open the Rawr save file (.xml) in a text editor\n  2) Select all of the code\n  3) Paste the code into the window below (May take several seconds)\n  4) Select which spec the items are for and click the 'Import' button at the bottom"

local EquipLocationNumbers = {
	{"INVTYPE_HEAD", 1, "Head"},
	{"INVTYPE_NECK", 2, "Neck"},
	{"INVTYPE_SHOULDER", 3, "Shoulder"},
	{"INVTYPE_CLOAK", 4, "Back"},
	{"INVTYPE_CHEST", 5, "Chest"},
	{"INVTYPE_ROBE", 5, "Chest"},
	{"INVTYPE_WRIST", 6, "Wrist"},
	{"INVTYPE_HAND", 7, "Hand"},
	{"INVTYPE_WAIST", 8, "Waist"},
	{"INVTYPE_LEGS", 9, "Leg"},
	{"INVTYPE_FEET", 10, "Foot"},
	{"INVTYPE_FINGER", 11, "Ring"},
	{"INVTYPE_TRINKET", 12, "Trinket"},
	{"INVTYPE_WEAPONMAINHAND", 13, "Main"},
	{"INVTYPE_2HWEAPON", 13, "Main"},
	{"INVTYPE_WEAPONOFFHAND", 14, "Off"},
	{"INVTYPE_HOLDABLE", 14, "Off"},
	{"INVTYPE_SHIELD", 14, "Off"},
	{"INVTYPE_RANGED", 15, "Ranged"},
	{"INVTYPE_THROWN", 15, "Ranged"},
	{"INVTYPE_RANGEDRIGHT", 15, "Ranged"},
	{"INVTYPE_RELIC", 15, "Ranged"},
} --Everything except INVTYPE_WEAPON, INVTYPE_BAG, INVTYPE_TABARD, and INVTYPE_BODY

function UpgradeList:RawrImport()
	InterfaceOptionsFrame:Hide()
	GameMenuFrame:Hide()
	if self.RawrImportFrame then
		self.RawrImportFrame:Show()
	else
		local frame = CreateFrame("frame", "RawrFrame", UIParent)
		frame:SetFrameStrata("DIALOG")
		frame:SetWidth(600)
		frame:SetHeight(700)
		frame:SetScale(1)
		
		frame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
			edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
			tile = true, tileSize = 32, edgeSize = 32, 
			insets = { left = 11, right = 11, top = 10, bottom = 10 }})

		frame:SetBackdropColor(0,0,0,0.8)

		frame:SetPoint("TOP",0,-80)
		
		frame:SetMovable(true)
		frame:EnableMouse(true)
		frame:RegisterForDrag("LeftButton")
		frame:SetScript("OnDragStart", frame.StartMoving)
		frame:SetScript("OnDragStop", frame.StopMovingOrSizing)
		
		tinsert(UISpecialFrames,"RawrFrame")

		frame.title = frame:CreateTexture(nil,"ARTWORK")
		frame.title:SetTexture("Interface/DialogFrame/UI-DialogBox-Header")
		frame.title:SetWidth(280)
		frame.title:SetHeight(64)
		frame.title:SetPoint("TOP", frame, 0, 12)
		frame.texture = frame.title

		frame.title.text = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		frame.title.text:SetText('Rawr Import')
		frame.title.text:SetPoint("TOP", frame, 0, -3)
		
		frame.closebutton = CreateFrame('Button', "CloseButton", frame, "UIPanelCloseButton")
		frame.closebutton:SetPoint("TOPRIGHT", -1, -2)
		frame.closebutton:SetWidth(40)
		frame.closebutton:SetHeight(40)
		
		frame.scrollframe = CreateFrame('ScrollFrame', "CodeWindow", frame)
		frame.scrollframe:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -120)
		frame.scrollframe:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -35, 80)
		frame.scrollframe:EnableMouseWheel(false)
		frame.scrollframe:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
			edgeFile = "Interface/GLUES/COMMON/TextPanel-Border",
			tile = true, tileSize = 32, edgeSize = 20, 
			insets = { left = 3, right = 3, top = 3, bottom = 3 }})
		frame.scrollframe:SetScript("OnMouseWheel", function(self, value)
			local height, viewheight = frame.scrollframe:GetHeight(), frame.content:GetHeight()
			local diff = height - viewheight
			local delta = 1
			if value < 0 then
				delta = -1
			end
			frame.scrollbar:SetValue(min(max(value + delta*(1000/(diff/45)),0), 1000))
		end)
		
		frame.scrollbar = CreateFrame('Slider', "Scroll Bar", frame.scrollframe, "UIPanelScrollBarTemplate")
		frame.scrollbar:SetPoint("TOPLEFT", frame.scrollframe, "TOPRIGHT", 4, -16)
		frame.scrollbar:SetPoint("BOTTOMLEFT", frame.scrollframe, "BOTTOMRIGHT", 4, 16)
		frame.scrollbar:SetMinMaxValues(0, 1000)
		frame.scrollbar:SetValueStep(1)
		frame.scrollbar:SetValue(0)
		frame.scrollbar:SetWidth(16)
		frame.scrollbar:SetScript("OnValueChanged", function(self, value)
			local viewheight = frame.scrollframe:GetHeight()
			local height = frame.content:GetHeight()
			if viewheight > height then
				offset = 0
			else
				offset = floor((height - viewheight) / 1000.0 * value)
			end
			frame.content:ClearAllPoints()
			frame.content:SetPoint("TOPLEFT", 0, offset)
			frame.content:SetPoint("TOPRIGHT", 0, offset)
		end)
		
		frame.content = CreateFrame('Editbox', "Code Entry", frame.scrollframe)
		frame.content:SetAllPoints()
		frame.content:SetFontObject(ChatFontNormal)
		frame.content:SetMultiLine(true)
		frame.content:EnableMouse(true)
		frame.content:SetAutoFocus(true)
		frame.content:SetCountInvisibleLetters(false)
		frame.content:SetTextInsets(6,5,5,5)
		frame.content:SetScript("OnTextChanged", function(self)
			if frame.content:GetText() == "" then
				frame.button:Disable()
			else
				frame.button:Enable()
			end
		end)
		
		frame.scrollframe:SetScrollChild(frame.content)
		
		frame.heading = CreateFrame('Frame', "Explanation", frame)
		frame.heading:SetPoint("TOPLEFT", frame, "TOPLEFT", 15, -20)
		frame.heading:SetPoint("BOTTOMRIGHT", frame.scrollframe, "TOPRIGHT")
		frame.heading.text = frame.heading:CreateFontString("ExplanationHeading", "ARTWORK", "GameFontNormal")
		frame.heading.text:SetText(RawrInputExplanation)
		frame.heading.text:SetAllPoints()
		frame.heading.text:SetJustifyH("LEFT")
		
		frame.betawarn = CreateFrame('Frame', "Beta Warning", frame)
		frame.betawarn:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 60)
		frame.betawarn:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, 5)
		frame.betawarn.text = frame.betawarn:CreateFontString("Beta Warn Text", "ARTWORK", "GameFontRedLarge")
		frame.betawarn.text:SetText("WARNING: This feature is currently in beta!  The scroll window does NOT scroll correctly.  You may need to scroll it down & up to see your code.")
		frame.betawarn.text:SetAllPoints()
		frame.betawarn.text:SetJustifyH("CENTER")
		
		frame.button = CreateFrame('Button', "Import Button", frame, "UIPanelButtonTemplate")
		frame.button:SetText("Import!")
		frame.button:SetWidth(100)
		frame.button:SetHeight(24)
		frame.button:SetPoint("BOTTOM", frame, "BOTTOM", 0, 30)
		frame.button:SetScript("OnClick", function(self, button, up)
			UpgradeList:PerformRawrImport(frame.content:GetText())
		end)
		frame.button:Disable()
		
		frame.PrimarySpecRadioButton = CreateFrame('CheckButton', "ImportPrimarySpecRadioButton", frame, "UIRadioButtonTemplate")
		frame.PrimarySpecRadioButton:SetHeight(20)
		frame.PrimarySpecRadioButton:SetWidth(20)
		frame.PrimarySpecRadioButton:SetChecked(true)
		frame.PrimarySpecRadioButton:SetPoint("BOTTOM", frame, "BOTTOM", -125, 60)
		_G[frame.PrimarySpecRadioButton:GetName() .. "Text"]:SetText(L["Primary Spec"])
		frame.PrimarySpecRadioButton:SetScript("OnClick", function (self, button, down)
			if GetNumTalentGroups("player") == 1 then
				SecondarySpecRadioButton:Disable()
				frame.PrimarySpecRadioButton:SetChecked(true)
			else
				frame.SecondarySpecRadioButton:SetChecked(false)
			end
		frame.PrimarySpecRadioButton:SetChecked(true)
		end)
		
		frame.SecondarySpecRadioButton = CreateFrame('CheckButton', "ImportSecondarySpecRadioButton", frame, "UIRadioButtonTemplate")
		frame.SecondarySpecRadioButton:SetHeight(20)
		frame.SecondarySpecRadioButton:SetWidth(20)
		frame.SecondarySpecRadioButton:SetChecked(false)
		frame.SecondarySpecRadioButton:SetPoint("BOTTOM", frame, "BOTTOM", 50, 60)
		_G[frame.SecondarySpecRadioButton:GetName() .. "Text"]:SetText(L["Secondary Spec"])
		if GetNumTalentGroups("player") == 1 then
			_G[frame.SecondarySpecRadioButton:GetName() .. "Text"]:SetTextColor(0.5, 0.5, 0.5)
			frame.SecondarySpecRadioButton:Disable()
		end
		frame.SecondarySpecRadioButton:SetScript("OnClick", function (self, button, down)
			frame.SecondarySpecRadioButton:SetChecked(true)
			frame.PrimarySpecRadioButton:SetChecked(false)
		end)
				
		self.RawrImportFrame = frame
		self.RawrImportFrame:Show()
	end
end

function UpgradeList:ReloadItem(id)
	GetItemInfo(id)
end

function UpgradeList:HandleItemsNotInJournal(itemsNotInJournalTable)
	for i = 1, #itemsNotInJournalTable, 1 do
		if itemsNotInJournalTable[i] == nil then
			table.remove(itemsNotInJournalTable, i)
		end
	end
	UpgradeList:ChatFrameMessage("   " .. #itemsNotInJournalTable .. " item(s) were not found in the Dungeon Journal.  The item(s) have been added to your lists, but the drop location is \"UNKNOWN\".  You may wish to manually re-add these items if you want to have the drop location listed.")
end

function UpgradeList:CheckJournal(idTable)
	local itemsNotInJournal = {}
	local foundValidItems = false
	for i = #idTable, 1, -1 do
		local itemName, itemLink, _, itemLevel, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(idTable[i])
		if itemLink ~= nil then
			foundValidItems = true
			EJ_SetSearch(itemName)
			if EJ_GetNumSearchResults() == 0 then --no match in dungeon journal
				itemsNotInJournal[i] = idTable[i]
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = "UNKNOWN"
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = "(From Rawr Import)"
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = ""
				local _, englishClass = UnitClass("player")
				if UpgradeList:ItemCanBeWorn(itemSubType, englishClass) then
					UpgradeList:ImportItemsIntoDatabase(self.RawrImportFrame.PrimarySpecRadioButton, itemLink, itemLevel, itemType, itemSubType, itemEquipLoc, idTable[i])
				end
			else
				for j = 1, EJ_GetNumSearchResults(), 1 do
					local EJID, _, EJDifficulty, EJInstanceID, EJEncounterID = EJ_GetSearchResult(j)
					local _, _, _, _, EJItemID = EJ_GetLootInfo(EJID)
					if tostring(EJItemID) == tostring(idTable[i]) then --match found
						UpgradeListDBVar.char.SavedOptions.SelectedInstance = UpgradeList.UI[EJInstanceID]
						UpgradeListDBVar.char.SavedOptions.SelectedBoss = UpgradeList.UI[EJEncounterID]
						UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = UpgradeList:DetermineDifficulty(EJDifficulty)
						local _, englishClass = UnitClass("player")	
						if UpgradeList:ItemCanBeWorn(itemSubType, englishClass) then
							UpgradeList:ImportItemsIntoDatabase(self.RawrImportFrame.PrimarySpecRadioButton, itemLink, itemLevel, itemType, itemSubType, itemEquipLoc, idTable[i])
						end
						break
					end
				end
			end
		end
	end
	if foundValidItems then
		UpgradeList:ChatFrameMessage("Import complete!")
		if #itemsNotInJournal > 0 then
			UpgradeList:HandleItemsNotInJournal(itemsNotInJournal)
		end
	end
end

function UpgradeList:ImportItemsIntoDatabase(PrimarySpecRadioButton, itemLink, itemLevel, itemType, itemSubType, itemEquipLoc, itemID)
	local AddToSpec
	local AddToSpecLong
	if PrimarySpecRadioButton:GetChecked() then
		AddToSpec = L["Primary"]
		AddToSpecLong = "PrimarySpecList"
	else
		AddToSpec = L["Secondary"]
		AddToSpecLong = "SecondarySpecList"
	end
	if itemEquipLoc ~= "" and itemEquipLoc ~= "INVTYPE_BAG" and itemEquipLoc ~= "INVTYPE_TABARD" and itemEquipLoc ~= "INVTYPE_BODY" then
		for i = 1, #EquipLocationNumbers, 1 do
			if itemEquipLoc == EquipLocationNumbers[i][1] then
				UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Row[UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count] = {UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, "", itemID }
				UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count = UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count + 1
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil
			end
		end
		if itemEquipLoc == "INVTYPE_WEAPON" then --Adding 1-H weapons to both Main Hand & Off Hand lists
			UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[13][3]].Row[UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[13][3]].Count] = {UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[13][3]].Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, "", itemID }
			UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[13][3]].Count = UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[13][3]].Count + 1
			
			UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[14][3]].Row[UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[14][3]].Count] = {UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[14][3]].Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, "", itemID }
			UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[14][3]].Count = UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[14][3]].Count + 1
			
			UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil
			UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil
			UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil
		end
	else
		UpgradeListDBVar.char[AddToSpecLong].Other.Row[UpgradeListDBVar.char[AddToSpecLong].Other.Count] = { UpgradeListDBVar.char[AddToSpecLong].Other.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, "", itemID }
		UpgradeListDBVar.char[AddToSpecLong].Other.Count = UpgradeListDBVar.char[AddToSpecLong].Other.Count + 1
		UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil
		UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil
		UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil
	end
end

function UpgradeList:PerformRawrImport(text)
	local foundItemIDs = {}
	local counter = 1
	for word, other in string.gmatch(text, "<AvailableItems>%d+</AvailableItems>") do
		local temp1 = string.gsub(word, "<AvailableItems>", "")
		local temp2 = string.gsub(temp1, "</AvailableItems>", "")
		foundItemIDs[counter] = tonumber(temp2)
		GetItemInfo(foundItemIDs[counter])
		counter = counter + 1
	end
	
	counter = counter - 1
	
	if counter == 0 then
		UpgradeList:ChatFrameMessage("No valid items found for import!")
	else
		for i = 1, counter, 1 do
			UpgradeList:ScheduleTimer("ReloadItem", 1, foundItemIDs[i])
			if i == counter then
				UpgradeList:ScheduleTimer("CheckJournal", 2, foundItemIDs)
			end
		end
	end
end