local mod = UpgradeList:NewModule("UI")
local L = LibStub("AceLocale-3.0"):GetLocale("UpgradeList")

local UPGRADELIST_VERSION = GetAddOnMetadata('UpgradeList', 'Version')

local Reputations = {}

function UpgradeList:PassRepsToUI(RepNames)
	Reputations = RepNames
end

local statusbar
local statusbarfont
UpgradeList.UI = {}

local TabNames = {
	{1, "Head", L["Head"]},
	{2, "Neck", L["Neck"]},
	{3, "Shoulder", L["Shoulders"]},
	{4, "Back", L["Back"]},
	{5, "Chest", L["Chest"]},
	{6, "Wrist", L["Wrists"]},
	{7, "Hand", L["Hands"]},
	{8, "Waist", L["Waist"]},
	{9, "Leg", L["Legs"]},
	{10, "Foot", L["Feet"]},
	{11, "Ring", L["Rings"]},
	{12, "Trinket", L["Trinkets"]},
	{13, "Main", L["Main Hand"]},
	{14, "Off", L["Off Hand"]},
	{15, "Ranged", L["Ranged"]},
	{16, "Other", L["Other"]},
}

function UpgradeList:DisableInput()
	UpgradeListMainFrame:EnableMouse(false)
	CloseButton:EnableMouse(false)
	ToggleButton:EnableMouse(false)
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(false)
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(false)
	OptionsFrame:EnableMouse(false)
	ItemEntryBox:EnableMouse(false)
	ULLocationDropDown_Button:EnableMouse(false)
	NoteEntryBox:EnableMouse(false)
	PrimarySpecRadioButton:EnableMouse(false)
	SecondarySpecRadioButton:EnableMouse(false)
	priorityfield:EnableMouse(false)
	AddButton:EnableMouse(false)
	tofield:EnableMouse(false)
	fromfield:EnableMouse(false)
	MoveButton:EnableMouse(false)
	DeleteButton:EnableMouse(false)
	deletefield:EnableMouse(false)
	multifromfield:EnableMouse(false)
	multitofield:EnableMouse(false)
	MultiDeleteButton:EnableMouse(false)
end

function UpgradeList:EnableInput()
	UpgradeListMainFrame:EnableMouse(true)
	CloseButton:EnableMouse(true)
	ToggleButton:EnableMouse(true)
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
	OptionsFrame:EnableMouse(true)
	ItemEntryBox:EnableMouse(true)
	ULLocationDropDown_Button:EnableMouse(true)
	NoteEntryBox:EnableMouse(true)
	PrimarySpecRadioButton:EnableMouse(true)
	SecondarySpecRadioButton:EnableMouse(true)
	priorityfield:EnableMouse(true)
	AddButton:EnableMouse(true)
	tofield:EnableMouse(true)
	fromfield:EnableMouse(true)
	MoveButton:EnableMouse(true)
	DeleteButton:EnableMouse(true)
	deletefield:EnableMouse(true)
	multifromfield:EnableMouse(true)
	multitofield:EnableMouse(true)
	MultiDeleteButton:EnableMouse(true)
end

function UpgradeList:StatusBarUpdate(message, color)
	if color == "red" then
		statusbarfont:SetTextColor(1, 0, 0)
	elseif color == "green" then
		statusbarfont:SetTextColor(0, 1, 0)
	else
		statusbarfont:SetTextColor(1, 1, 1)
	end
	statusbarfont:SetText(message)
end

function UpgradeList:ChatFrameMessage(message)
	DEFAULT_CHAT_FRAME:AddMessage("|cFFE08800Upgrade List:|r " .. message)
end

function UpgradeList:ScrollingWindowMessage(message)
	UpgradeListDBVar.profile.messages.scrollwindow:AddMessage(message)
end

function UpgradeList:UpdateIcon(icon)
	if icon == true then
		self.UI.AutoTexture:SetTexture("Interface\\RAIDFRAME\\ReadyCheck-Ready.blp")
	else
		self.UI.AutoTexture:SetTexture("Interface\\RAIDFRAME\\ReadyCheck-NotReady.blp")
	end
end

function UpgradeList:LoadDungeonJournalInfo()
	local instanceCount = 1
	local bossCount = 1
	EJ_SetDifficulty(true, true)
	repeat
		local instanceID, name = EJ_GetInstanceByIndex(instanceCount, false)
		UpgradeList.UI[instanceID] = name
		EJ_SelectInstance(instanceID)
		bossCount = 1
		repeat
			local bossName, _, bossID = EJ_GetEncounterInfoByIndex(bossCount)
			UpgradeList.UI[bossID] = bossName
			bossCount = bossCount + 1
		until EJ_GetEncounterInfoByIndex(bossCount) == nil
		instanceCount = instanceCount + 1
	until EJ_GetInstanceByIndex(instanceCount, false) == nil
	instanceCount = 1
	repeat
		local raidID, name = EJ_GetInstanceByIndex(instanceCount, true)
		UpgradeList.UI[raidID] = name
		EJ_SelectInstance(raidID)
		bossCount = 1
		repeat
			local bossName, _, raidBossID = EJ_GetEncounterInfoByIndex(bossCount)
			UpgradeList.UI[raidBossID] = bossName
			bossCount = bossCount + 1
		until EJ_GetEncounterInfoByIndex(bossCount) == nil
		instanceCount = instanceCount + 1
	until EJ_GetInstanceByIndex(instanceCount, true) == nil
	EJ_SetDifficulty(false, true)
end

function UpgradeList:DetermineDifficulty(mask)
	if tostring(mask) == "12" then
		return "Heroic"
	else
		return "Normal"
	end
end

function UpgradeList:CreateScrollingWindow()
	UpgradeListDBVar.profile.messages.scrollwindow = CreateFrame("ScrollingMessageFrame", "Scrolling Window", UIParent, "ChatFrameTemplate")
	UpgradeListDBVar.profile.messages.scrollwindow:SetPoint("CENTER", 0, 200)
	--[[
	UpgradeListDBVar.profile.messages.scrollwindow:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }})
	--]]
	UpgradeListDBVar.profile.messages.scrollwindow:SetWidth(UIParent:GetWidth() - 200)
	UpgradeListDBVar.profile.messages.scrollwindow:SetHeight(50)
	UpgradeListDBVar.profile.messages.scrollwindow:SetFadeDuration(2)
	UpgradeListDBVar.profile.messages.scrollwindow:SetFading(true)
	UpgradeListDBVar.profile.messages.scrollwindow:SetHyperlinksEnabled(true)
	UpgradeListDBVar.profile.messages.scrollwindow:SetInsertMode("top")
	UpgradeListDBVar.profile.messages.scrollwindow:SetMaxLines(5)
	UpgradeListDBVar.profile.messages.scrollwindow:SetTimeVisible(10)
	UpgradeListDBVar.profile.messages.scrollwindow:SetJustifyH("CENTER")
	UpgradeListDBVar.profile.messages.scrollwindow:SetJustifyV("CENTER")
	UpgradeListDBVar.profile.messages.scrollwindow:SetFontObject(GameFontNormalHuge)
	UpgradeListDBVar.profile.messages.scrollwindow:SetTextColor(1, 1, 1) --white
	UpgradeListDBVar.profile.messages.scrollwindow:EnableMouse(false)
	UpgradeListDBVar.profile.messages.scrollwindow:Show()
end

function UpgradeList:CreateMainFrame()
	local UpgradeListMainFrame = CreateFrame("Frame","UpgradeListMainFrame",UIParent)
	UpgradeListMainFrame:SetFrameStrata("DIALOG")
	UpgradeListMainFrame:SetWidth(1257)
	UpgradeListMainFrame:SetHeight(280)
	UpgradeListMainFrame:SetScale(UpgradeListDBVar.profile.mainframe.scale)
	
	UpgradeListMainFrame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }})

	UpgradeListMainFrame:SetBackdropColor(0,0,0,1)

	UpgradeListMainFrame:SetPoint("TOP",0,-80)
	
	--Makes the Main Frame (and therefore all frames attached to it) draggable
	if UpgradeListDBVar.profile.mainframe.lock == false then
		UpgradeListMainFrame:SetMovable(true)
		UpgradeListMainFrame:EnableMouse(true)
	end
	UpgradeListMainFrame:RegisterForDrag("LeftButton")
	UpgradeListMainFrame:SetScript("OnDragStart", UpgradeListMainFrame.StartMoving)
	UpgradeListMainFrame:SetScript("OnDragStop", UpgradeListMainFrame.StopMovingOrSizing)
	
	tinsert(UISpecialFrames,"UpgradeListMainFrame")


	--Make a title section for the Main Frame 
	local MF_titleBG = UpgradeListMainFrame:CreateTexture(nil,"ARTWORK")
	MF_titleBG:SetTexture("Interface/DialogFrame/UI-DialogBox-Header")
	MF_titleBG:SetWidth(280)
	MF_titleBG:SetHeight(64)
	MF_titleBG:SetPoint("TOP", UpgradeListMainFrame, 0, 12)
	UpgradeListMainFrame.texture = MF_titleBG

	local MF_titleText = UpgradeListMainFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	MF_titleText:SetText('Upgrade List')
	MF_titleText:SetPoint("TOP", UpgradeListMainFrame, 0, -3)
	
	--Version Identification
	VersionHeading = CreateFrame('Frame', "VersionHeading", UpgradeListMainFrame)
	VersionHeadingFont = VersionHeading:CreateFontString("VersionHeading", "ARTWORK", "GameFontWhiteTiny")
	VersionHeadingFont:SetText("v " .. UPGRADELIST_VERSION)
	VersionHeadingFont:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 13, -11)
	VersionHeadingFont:SetWidth(VersionHeadingFont:GetStringWidth() + 5)
	VersionHeadingFont:SetHeight(VersionHeadingFont:GetStringHeight())
	
	--Make an invisible frame within the Main Frame that will contain the UpgradeListDBVar.char.SavedOptions.tableUL
	local TableContainer = CreateFrame("Frame", "TableContainer", UpgradeListMainFrame)
	TableContainer:SetWidth(1265)
	TableContainer:SetHeight(235)
	TableContainer:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 0, -30)
	TableContainer:SetPoint("BOTTOMRIGHT", UpgradeListMainFrame, "BOTTOMRIGHT", 0, -12)
	return UpgradeListMainFrame, MF_titleBG, MF_titleText, VersionHeading, TableContainer
end

function UpgradeList:CreateScrollingItemFrame()
	local cols = {
		{  -- col 1
			name	= L["Priority"],
			width	= 42,
			defaultsort = "asc",
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 2
			name	= L["iLvl"],
			width	= 35,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 3
			name	= L["Item"],
			width	= 240,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 4
			name	= L["Difficulty"],
			width	= 80,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 5
			name	= L["Instance/Reputation/Point Type/Token"],
			width	= 250,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 6
			name	= L["Boss/Reputation Level/Points Needed/Token Location"],
			width	= 280,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 7
			name	= L["Note"],
			width	= 280,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
	}

	local rows = 14
	local height = 15
	local UpgradeListScrollingTable = LibStub("ScrollingTable")
	UpgradeListDBVar.char.SavedOptions.tableUL = UpgradeListScrollingTable:CreateST(cols, rows, height, nil, TableContainer) --UpgradeListDBVar.char.SavedOptions.tableUL
	UpgradeListDBVar.char.SavedOptions.tableUL.frame:SetPoint("CENTER", TableContainer, "CENTER", 0, 0)
	UpgradeListDBVar.char.SavedOptions.tableUL:RegisterEvents({
		["OnClick"] = function (rowFrame, cellFrame, data, cols, row, realrow, column, UpgradeListScrollingTable, ...)
			ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
			if row ~= nil then --if the user isn't clicking in the column header area
				if IsModifiedClick("CTRL") then
					EditNote:Show()
					UpgradeList:DisableInput()
				end
				local ShownSpec
				local ShownSpecLong
				if UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() then
					ShownSpec = L["Primary"]
					ShownSpecLong = "PrimarySpecList"
				else
					ShownSpec = L["Secondary"]
					ShownSpecLong = "SecondarySpecList"
				end
				for i = 1, #TabNames, 1 do
					if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == TabNames[i][1] then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText()
								EditNote:Hide()
								EditNoteBox:SetText("")
								UpgradeListDBVar.char[ShownSpecLong][TabNames[i][2]].Row[realrow][7] = newnote
								UpgradeList:EnableInput()
								UpgradeList:RefreshWindow()
							end)
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char[ShownSpecLong][TabNames[i][2]].Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char[ShownSpecLong][TabNames[i][2]].Row[realrow][3])
						end
					end
				end
			end
			ShowUIPanel(ItemRefTooltip)
		end,
	})
end

function UpgradeList:CloseButton()
	local CloseButton = CreateFrame('Button', "CloseButton", UpgradeListMainFrame, "UIPanelCloseButton")
	CloseButton:SetPoint("TOPRIGHT", -1, -2)
	CloseButton:SetWidth(40)
	CloseButton:SetHeight(40)
	return CloseButton
end

function UpgradeList:ToggleOptionsButton()
	local ToggleButton = CreateFrame('Button', "ToggleButton", UpgradeListMainFrame, "UIPanelButtonTemplate2")
	ToggleButton:SetPoint("RIGHT", CloseButton, "LEFT", 8, 0)
	ToggleButton:SetWidth(170)
	ToggleButton:SetHeight(20)
	ToggleButton:SetText(L["Toggle Options Window"])
	ToggleButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:ToggleOptions()
	end)
	return ToggleButton
end

function UpgradeList:PrimarySpecViewButton()
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck = CreateFrame('CheckButton', "UpgradeListDBVar.char.SavedOptions.PrimaryCheck", UpgradeListMainFrame, "UICheckButtonTemplate")
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetPoint("RIGHT", ToggleButton, "LEFT", -275, -2)
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true)
	_G[UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetName() .. "Text"]:SetText(L["View Primary Spec"])
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetScript("OnClick", function (self, button, down)
		if GetNumTalentGroups("player") == 1 then
			UpgradeListDBVar.char.SavedOptions.SecondaryCheck:Disable()
			UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true)
		else
			UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(false)
			UpgradeList:RefreshWindow()
		end
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true)
	end)
end

function UpgradeList:SecondarySpecViewButton()
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck = CreateFrame('CheckButton', "UpgradeListDBVar.char.SavedOptions.SecondaryCheck", UpgradeListMainFrame, "UICheckButtonTemplate")
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetPoint("RIGHT", ToggleButton, "LEFT", -125, -2)
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(false)
	_G[UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetName() .. "Text"]:SetText(L["View Secondary Spec"])
	if GetNumTalentGroups("player") == 1 then
		_G[UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetName() .. "Text"]:SetTextColor(0.5, 0.5, 0.5)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:Disable()
	end
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetScript("OnClick", function (self, button, down)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(true)
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(false)
		UpgradeList:RefreshWindow()
	end)
end

function UpgradeList:ExplanationHeadingText()
	local ExplanationHeading = CreateFrame('Frame', "ExplanationHeading", UpgradeListMainFrame)
	local ExplanationHeadingFont = ExplanationHeading:CreateFontString("ExplanationHeading", "ARTWORK", "GameFontWhiteTiny")
	ExplanationHeadingFont:SetText(L["Clicking a row will show the tooltip for the item, Shift-clicking a row will allow you to add the item link into your chat window, Ctrl-clicking a row will allow you to edit the note for that row."])
	ExplanationHeadingFont:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 50, -11)
	ExplanationHeadingFont:SetWidth(500)
	ExplanationHeadingFont:SetHeight(ExplanationHeadingFont:GetStringHeight())
	ExplanationHeadingFont:SetJustifyH("LEFT")
	return ExplanationHeading
end

function UpgradeList:TabCreate()
	PanelTemplates_SetNumTabs(UpgradeListMainFrame, 16)
	Tab_1 = CreateFrame('Button', "$parentTab1", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_1:SetID(1)
	Tab_1:SetText(L['Head'])
	Tab_1:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -575, 6)
	PanelTemplates_TabResize(Tab_1, 0, 93, 94)
	Tab_1:RegisterForClicks("AnyUp")
	Tab_1:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_1, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)

	Tab_2 = CreateFrame('Button', "$parentTab2", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_2:SetID(2)
	Tab_2:SetText(L['Neck'])
	Tab_2:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -499, 6)
	PanelTemplates_TabResize(Tab_2, 0, 93, 94)
	Tab_2:RegisterForClicks("AnyUp")
	Tab_2:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_2, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_3 = CreateFrame('Button', "$parentTab3", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_3:SetID(3)
	Tab_3:SetText(L['Shoulders'])
	Tab_3:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -423, 6)
	PanelTemplates_TabResize(Tab_3, 0, 93, 94)
	Tab_3:RegisterForClicks("AnyUp")
	Tab_3:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_3, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_4 = CreateFrame('Button', "$parentTab4", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_4:SetID(4)
	Tab_4:SetText(L['Back'])
	Tab_4:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -347, 6)
	PanelTemplates_TabResize(Tab_4, 0, 93, 94)
	Tab_4:RegisterForClicks("AnyUp")
	Tab_4:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_4, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_5 = CreateFrame('Button', "$parentTab5", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_5:SetID(5)
	Tab_5:SetText(L['Chest'])
	Tab_5:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -271, 6)
	PanelTemplates_TabResize(Tab_5, 0, 93, 94)
	Tab_5:RegisterForClicks("AnyUp")
	Tab_5:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_5, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_6 = CreateFrame('Button', "$parentTab6", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_6:SetID(6)
	Tab_6:SetText(L['Wrists'])
	Tab_6:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -195, 6)
	PanelTemplates_TabResize(Tab_6, 0, 93, 94)
	Tab_6:RegisterForClicks("AnyUp")
	Tab_6:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_6, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_7 = CreateFrame('Button', "$parentTab7", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_7:SetID(7)
	Tab_7:SetText(L['Hands'])
	Tab_7:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -119, 6)
	PanelTemplates_TabResize(Tab_7, 0, 93, 94)
	Tab_7:RegisterForClicks("AnyUp")
	Tab_7:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_7, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_8 = CreateFrame('Button', "$parentTab8", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_8:SetID(8)
	Tab_8:SetText(L['Waist'])
	Tab_8:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -43, 6)
	PanelTemplates_TabResize(Tab_8, 0, 93, 94)
	Tab_8:RegisterForClicks("AnyUp")
	Tab_8:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_8, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_9 = CreateFrame('Button', "$parentTab9", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_9:SetID(9)
	Tab_9:SetText(L['Legs'])
	Tab_9:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 33, 6)
	PanelTemplates_TabResize(Tab_9, 0, 93, 94)
	Tab_9:RegisterForClicks("AnyUp")
	Tab_9:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_9, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_10 = CreateFrame('Button', "$parentTab10", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_10:SetID(10)
	Tab_10:SetText(L['Feet'])
	Tab_10:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 109, 6)
	PanelTemplates_TabResize(Tab_10, 0, 93, 94)
	Tab_10:RegisterForClicks("AnyUp")
	Tab_10:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_10, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_11 = CreateFrame('Button', "$parentTab11", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_11:SetID(11)
	Tab_11:SetText(L['Rings'])
	Tab_11:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 185, 6)
	PanelTemplates_TabResize(Tab_11, 0, 93, 94)
	Tab_11:RegisterForClicks("AnyUp")
	Tab_11:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_11, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_12 = CreateFrame('Button', "$parentTab12", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_12:SetID(12)
	Tab_12:SetText(L['Trinkets'])
	Tab_12:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 261, 6)
	PanelTemplates_TabResize(Tab_12, 0, 93, 94)
	Tab_12:RegisterForClicks("AnyUp")
	Tab_12:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_12, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_13 = CreateFrame('Button', "$parentTab13", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_13:SetID(13)
	Tab_13:SetText(L['Main Hand'])
	Tab_13:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 342, 6)
	PanelTemplates_TabResize(Tab_13, 0, 102, 90)
	Tab_13:RegisterForClicks("AnyUp")
	Tab_13:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_13, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_14 = CreateFrame('Button', "$parentTab14", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_14:SetID(14)
	Tab_14:SetText(L['Off Hand'])
	Tab_14:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 423, 6)
	PanelTemplates_TabResize(Tab_14, 0, 94, 94)
	Tab_14:RegisterForClicks("AnyUp")
	Tab_14:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_14, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_15 = CreateFrame('Button', "$parentTab15", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_15:SetID(15)
	Tab_15:SetText(L['Ranged'])
	Tab_15:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 499, 6)
	PanelTemplates_TabResize(Tab_15, 0, 94, 94)
	Tab_15:RegisterForClicks("AnyUp")
	Tab_15:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_15, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	Tab_16 = CreateFrame('Button', "$parentTab16", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate")
	Tab_16:SetID(16)
	Tab_16:SetText(L['Other'])
	Tab_16:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 575, 6)
	PanelTemplates_TabResize(Tab_16, 0, 94, 94)
	Tab_16:RegisterForClicks("AnyUp")
	Tab_16:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID())
		PanelTemplates_Tab_OnClick(Tab_16, UpgradeListMainFrame)
		UpgradeList:RefreshWindow()
	end)
	
	PanelTemplates_SetTab(UpgradeListMainFrame, 1)
	UpgradeList:RefreshWindow()
end

function UpgradeList:CreateOptionsFrame()
	local OptionsFrame = CreateFrame("Frame", "OptionsFrame", UpgradeListMainFrame)
	OptionsFrame:SetWidth(850)
	OptionsFrame:SetHeight(330)
	OptionsFrame:SetPoint("TOP",UpgradeListMainFrame,"BOTTOM", 0, -30)
	OptionsFrame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 12, top = 12, bottom = 11 }})
	return OptionsFrame
end

function UpgradeList:CreateItemEntryBox()
	local ItemEntryBox = CreateFrame('EditBox', "ItemEntryBox", OptionsFrame, "InputBoxTemplate")
	ItemEntryBox:SetPoint("CENTER", 0, 125)
	ItemEntryBox:SetAutoFocus(false)
	ItemEntryBox:SetWidth(130)
    ItemEntryBox:SetHeight(24)
	ItemEntryBox:SetScript("OnTextChanged", function (self, userInput)
		if userInput then
			UpgradeList:CancelAllTimers()
			UpgradeList:UpdateIcon(false)
			if GetItemInfo(ItemEntryBox:GetText()) == nil then
				UpgradeList:ScheduleRepeatingTimer("ValidateItem", 0.5, ItemEntryBox:GetText()) --keeps checking for new items loaded into cache
			else --item exists, now have to check if it's in EJ
				local itemName, itemLink = GetItemInfo(ItemEntryBox:GetText()) 
				local _, _, _, _, itemID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
				UpgradeList:CancelAllTimers()
				EJ_SetSearch(itemName)
				if EJ_GetNumSearchResults() == 0 then --no match in dungeon journal
					UpgradeList:UpdateIcon(false)
					UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Location:"])
				else
					for i = 1, EJ_GetNumSearchResults(), 1 do
						local EJID, _, EJDifficulty, EJInstanceID, EJEncounterID = EJ_GetSearchResult(i)
						local _, _, _, _, EJItemID = EJ_GetLootInfo(EJID)
						if tostring(EJItemID) == tostring(itemID) then --match found
							UpgradeList:UpdateIcon(true) --catches items already in cache
							UpgradeListDBVar.char.SavedOptions.SelectedInstance = UpgradeList.UI[EJInstanceID]
							UpgradeListDBVar.char.SavedOptions.SelectedBoss = UpgradeList.UI[EJEncounterID]
							UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = UpgradeList:DetermineDifficulty(EJDifficulty)
							UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Location Auto-Set!"])
							break
						end
					end
				end
			end
		end
	end)
	ItemEntryBox:SetScript("OnHide", function(self)
		UpgradeList:CancelAllTimers()
	end)
	
	local AutoItemIndicator = CreateFrame('Frame', "AutoItemIndicator", OptionsFrame)
	AutoItemIndicator:SetHeight(ItemEntryBox:GetHeight() - 5)
	AutoItemIndicator:SetWidth(AutoItemIndicator:GetHeight())
	AutoItemIndicator:SetPoint("LEFT", ItemEntryBox, "RIGHT", 5, 0)
	local AutoTexture = AutoItemIndicator:CreateTexture("AutoTexture", "HIGH")
	AutoTexture:SetAllPoints(AutoItemIndicator)
	AutoTexture:SetTexture("Interface\\RAIDFRAME\\ReadyCheck-NotReady.blp")
	
	local ItemEntryBoxHeading = CreateFrame('Frame', "ItemEntryBoxHeading", OptionsFrame)
	local HeadingFont = ItemEntryBoxHeading:CreateFontString("ItemHeading", "ARTWORK", "GameFontNormalSmallLeft")
	HeadingFont:SetText(L["Enter your item with an item link or an item ID number:"])
	HeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP")
	HeadingFont:SetWidth(HeadingFont:GetStringWidth() + 5)
	HeadingFont:SetHeight(HeadingFont:GetStringHeight())	
	
	return ItemEntryBox, ItemEntryBoxHeading, AutoItemIndicator, AutoTexture
end

function UpgradeList:CreateDropdownMenu()
	CreateFrame("Button", "ULLocationDropDown_Button", OptionsFrame, "UIDropDownMenuTemplate")
	ULLocationDropDown_Button:ClearAllPoints()
	ULLocationDropDown_Button:SetPoint("TOP", ItemEntryBox, "BOTTOM", 0, -20)
	ULLocationDropDown_Button:Show()
	

	function DropDown_InitializeDropDown(self, level)

		--Top Level
		if level == 1 then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:TopLevel(info, level)

		--Raid Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Raid"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:RaidSubmenu(info, level)
			
		--Dungeon Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Dungeon"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:DungeonSubmenu(info, level)
			
		--Crafted Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Crafted"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:CraftedSubmenu(info, level)
			
		--Quest Reward Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Quest Reward"]) then
			local info = UIDropDownMenu_CreateInfo()
			--local DropLocation = "Quest Reward"
			
			info.text = L["Quest Reward"]
			info.value = L["Quest Reward"]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Quest Reward"])
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = L["Quest Reward"]
				CloseDropDownMenus(1)
			end
			UIDropDownMenu_AddButton(info, level)

		--Reputation Reward Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Reputation Reward"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Reputations(info, level)
		
		--Purchasable Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Purchasable"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Purchasable(info, level)
			
		--World Drop Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["World Drop"]) then
			local info = UIDropDownMenu_CreateInfo()
			
			info.text = L["World Drop"]
			info.value = L["World Drop"]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["World Drop"])
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = L["World Drop"]
				CloseDropDownMenus(1)
			end
			UIDropDownMenu_AddButton(info, level)

		--List of heroic raids
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Heroic"] .. "Raid") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HeroicRaids(info, level)
			
		--List of normal raids
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Normal"] .. "Raid") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:NormalRaids(info, level)
		
		--Heroic Dungeons
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Heroic"] .. "Dungeon") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HeroicDungeons(info, level)
			
		--Normal Dungeons
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Normal"] .. "Dungeon") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:NormalDungeons(info, level)
		
		--Reputation Levels
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == Reputations[1]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[2]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[3]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[4]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[5]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[6]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[7]
			or UIDROPDOWNMENU_MENU_VALUE == GetGuildInfo("player")) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:RepLevels(info, level, UIDROPDOWNMENU_MENU_VALUE)
		
		--Tier Selection for Tokens
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Gear Token"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:TierSelection(info, level)
			
		--Tier Tokens
		elseif (level == 4) and ((UIDROPDOWNMENU_MENU_VALUE == L["Tier 11"])
		or (UIDROPDOWNMENU_MENU_VALUE == L["Tier 12"])) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GearTokens(info, level, UIDROPDOWNMENU_MENU_VALUE)
			
		--Normal Firelands Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 5 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Firelands(info, level, L["Normal"])
			
		--Heroic Firelands Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 5 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Firelands(info, level, L["Heroic"])
			
		--Normal Bastion of Twilight Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 3 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BastionOfTwilight(info, level, L["Normal"])
		
		--Heroic Bastion of Twilight Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 3 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BastionOfTwilight(info, level, L["Heroic"])
			
		--Normal Blackwing Descent Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 4 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackwingDescent(info, level, L["Normal"])
			
		--Heroic Blackwing Descent Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 4 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackwingDescent(info, level, L["Heroic"])
		
		--Normal Throne of the Four Winds Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 2 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTheFourWinds(info, level, L["Normal"])
			
		--Heroic Throne of the Four Winds Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 2 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTheFourWinds(info, level, L["Heroic"])
			
		--Heroic Baradin Hold Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 1 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BaradinHold(info, level, L["Heroic"])
			
		--Normal Baradin Hold Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 1 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BaradinHold(info, level, L["Normal"])
			
		--Heroic Zul'Aman Bosses
		elseif (level==4) and (UIDROPDOWNMENU_MENU_VALUE == "2 4 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ZulAman(info, level)
			
		--Heroic Zul'Gurub Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "2 3 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ZulGurub(info, level)
		
		--Heroic Deadmines Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "2 2 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Deadmines(info, level)
			
		--Heroic Shadowfang Keep Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "2 1 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ShadowfangKeep(info, level)
			
		--Heroic Throne of Tides Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 2 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTides(info, level, L["Heroic"])
			
		--Heroic Blackrock Caverns Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 7 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackrockCaverns(info, level, L["Heroic"])
			
		--Heroic Stonecore Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 3 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Stonecore(info, level, L["Heroic"])

		--Heroic Vortex Pinnacle Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 1 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:VortexPinnacle(info, level, L["Heroic"])

		--Heroic Lost City of Tol'vir Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 4 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:LostCityOfTolvir(info, level, L["Heroic"])
			
		--Heroic Grim Batol Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 6 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GrimBatol(info, level, L["Heroic"])
			
		--Heroic Halls of Origination Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 5 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HallsOfOrigination(info, level, L["Heroic"])
			
		--Normal Throne of Tides Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 2 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTides(info, level, L["Normal"])
			
		--Normal Blackrock Caverns Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 7 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackrockCaverns(info, level, L["Normal"])
		
		--Normal Stonecore Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 3 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Stonecore(info, level, L["Normal"])
		
		--Normal Vortex Pinnacle Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 1 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:VortexPinnacle(info, level, L["Normal"])
		
		--Normal Lost City of Tol'vir Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 4 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:LostCityOfTolvir(info, level, L["Normal"])
		
		--Normal Grim Batol Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 6 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GrimBatol(info, level, L["Normal"])
		
		--Normal Halls of Origination Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 5 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HallsOfOrigination(info, level, L["Normal"])
		
		end
	end

	function DropDown_DropDownOnLoad(self)
		UIDropDownMenu_Initialize(self, DropDown_InitializeDropDown)
	end

	function ULLocationDropDown_ButtonOnClick(self, button, down)
		local name = self:GetName()
		ToggleDropDownMenu(1, nil, DropDown_DropDown, name, 0, 0)
	end
	
	UIDropDownMenu_Initialize(ULLocationDropDown_Button, DropDown_InitializeDropDown)
	UIDropDownMenu_SetWidth(ULLocationDropDown_Button, 175)
	UIDropDownMenu_SetButtonWidth(ULLocationDropDown_Button, 175)
	UIDropDownMenu_SetSelectedID(ULLocationDropDown_Button, 0)
	UIDropDownMenu_JustifyText(ULLocationDropDown_Button, "CENTER")
	UIDropDownMenu_SetText(ULLocationDropDown_Button, "Location:")

	local DropDownHeading = CreateFrame('Frame', "DropDownHeading", OptionsFrame)
	local NoteFont = DropDownHeading:CreateFontString("DropDownHeading", "ARTWORK", "GameFontNormalSmallLeft")
	NoteFont:SetText(L["Select Item Location:"])
	NoteFont:SetPoint("BOTTOM", ULLocationDropDown_Button, "TOP")
	NoteFont:SetWidth(NoteFont:GetStringWidth() + 5)
	NoteFont:SetHeight(NoteFont:GetStringHeight())	
	return DropDownHeading
end

function UpgradeList:CreateNoteEntryBox()
	local NoteEntryBox = CreateFrame('EditBox', "NoteEntryBox", OptionsFrame, "InputBoxTemplate")
	NoteEntryBox:SetPoint("TOP", ULLocationDropDown_Button, "BOTTOM", 0, -20)
	NoteEntryBox:SetAutoFocus(false)
	NoteEntryBox:SetWidth(250)
    NoteEntryBox:SetHeight(24)
	NoteEntryBox:SetMaxLetters(28)
	
	local NoteEntryBoxHeading = CreateFrame('Frame', "NoteEntryBoxHeading", OptionsFrame)
	local NoteFont = NoteEntryBoxHeading:CreateFontString("NoteHeading", "ARTWORK", "GameFontNormalSmallLeft")
	NoteFont:SetText(L["Enter an optional note:"])
	NoteFont:SetPoint("BOTTOM", NoteEntryBox, "TOP")
	NoteFont:SetWidth(NoteFont:GetStringWidth() + 5)
	NoteFont:SetHeight(NoteFont:GetStringHeight())	
	return NoteEntryBox, NoteEntryBoxHeading
end

function UpgradeList:CreateSpecRadioButtons()
	--Primary Spec Selector Radio Button
	local PrimarySpecRadioButton = CreateFrame('CheckButton', "PrimarySpecRadioButton", OptionsFrame, "UIRadioButtonTemplate")
	PrimarySpecRadioButton:SetHeight(20)
	PrimarySpecRadioButton:SetWidth(20)
	PrimarySpecRadioButton:SetChecked(true)
	PrimarySpecRadioButton:SetPoint("TOP", NoteEntryBox, "BOTTOM", -95, -25)
	_G[PrimarySpecRadioButton:GetName() .. "Text"]:SetText(L["Primary Spec"])
	PrimarySpecRadioButton:SetScript("OnClick", function (self, button, down)
		if GetNumTalentGroups("player") == 1 then
			SecondarySpecRadioButton:Disable()
			PrimarySpecRadioButton:SetChecked(true)
		else
			SecondarySpecRadioButton:SetChecked(false)
		end
	PrimarySpecRadioButton:SetChecked(true)
	end)
	
	--Secondary Spec Selector Radio Button
	local SecondarySpecRadioButton = CreateFrame('CheckButton', "SecondarySpecRadioButton", OptionsFrame, "UIRadioButtonTemplate")
	SecondarySpecRadioButton:SetHeight(20)
	SecondarySpecRadioButton:SetWidth(20)
	SecondarySpecRadioButton:SetChecked(false)
	SecondarySpecRadioButton:SetPoint("TOP", NoteEntryBox, "BOTTOM", 20, -25)
	_G[SecondarySpecRadioButton:GetName() .. "Text"]:SetText(L["Secondary Spec"])
	if GetNumTalentGroups("player") == 1 then
		_G[SecondarySpecRadioButton:GetName() .. "Text"]:SetTextColor(0.5, 0.5, 0.5)
		SecondarySpecRadioButton:Disable()
	end
	SecondarySpecRadioButton:SetScript("OnClick", function (self, button, down)
		SecondarySpecRadioButton:SetChecked(true)
		PrimarySpecRadioButton:SetChecked(false)
	end)
	
	local SpecEntryBoxHeading = CreateFrame('Frame', "SpecEntryBoxHeading", OptionsFrame)
	SpecFont = SpecEntryBoxHeading:CreateFontString("SpecHeading", "ARTWORK", "GameFontNormalSmallLeft")
	SpecFont:SetText(L["Select which spec to which you would like to add this item:"])
	SpecFont:SetPoint("BOTTOM", NoteEntryBox, "TOP", 0, -47)
	SpecFont:SetWidth(SpecFont:GetStringWidth() + 5)
	SpecFont:SetHeight(SpecFont:GetStringHeight())	
	return PrimarySpecRadioButton, SecondarySpecRadioButton
end

function UpgradeList:AddItemLocation()
	local priorityfield = CreateFrame('EditBox', "priorityfield", OptionsFrame, "InputBoxTemplate")
	priorityfield:SetPoint("TOP", ItemEntryBox, "BOTTOM", 0, -180)
	priorityfield:SetAutoFocus(false)
	priorityfield:SetWidth(25)
	priorityfield:SetHeight(24)
	
	local PriorityBoxHeading = CreateFrame('Frame', "PriorityBoxHeading", OptionsFrame)
	local PriorityFont = PriorityBoxHeading:CreateFontString("SpecHeading", "ARTWORK", "GameFontNormalSmallLeft")
	PriorityFont:SetText(L["Enter the priority number for this item OR \nleave it blank to add the item to bottom of the list:"])
	PriorityFont:SetJustifyH("CENTER")
	PriorityFont:SetPoint("BOTTOM", priorityfield, "TOP", 0, 5)
	PriorityFont:SetWidth(PriorityFont:GetStringWidth() + 5)
	PriorityFont:SetHeight(PriorityFont:GetStringHeight())	
	
	return priorityfield, PriorityBoxHeading
end

function UpgradeList:CreateAddButton()
	local AddButton = CreateFrame('Button', "AddButton", OptionsFrame, "UIPanelButtonTemplate")
	AddButton:SetPoint("TOP", priorityfield, "BOTTOM", 0, -5)
	AddButton:SetWidth(100)
    AddButton:SetHeight(24)
	AddButton:SetText(L["Add Item"])
	AddButton:SetScript("OnEnter", function (self, motion)
		if ItemEntryBox ~= "" then
			UpgradeList:LoadItemIntoCache(ItemEntryBox)
		end
	end)
	AddButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:AddItem(ItemEntryBox, PrimarySpecRadioButton, UpgradeListMainFrame, priorityfield, NoteEntryBox)
	end)
	return AddButton
end

function UpgradeList:CreateMoveItem()
	--Moving heading text
	local MovingHeading = CreateFrame('Frame', "MovingHeading", OptionsFrame)
	local MovingHeadingFont = MovingHeading:CreateFontString("MovingHeading", "ARTWORK", "GameFontNormalSmallLeft")
	MovingHeadingFont:SetText(L["In the currently selected tab, move item #:"])
	MovingHeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP", -275, 0)
	MovingHeadingFont:SetWidth(MovingHeadingFont:GetStringWidth() + 5)
	MovingHeadingFont:SetHeight(MovingHeadingFont:GetStringHeight())
	
	--"From" field for moving items
	local fromfield = CreateFrame('EditBox', "fromfield", OptionsFrame, "InputBoxTemplate")
	fromfield:SetPoint("TOP", MovingHeadingFont, "BOTTOM", -25, -2)
	fromfield:SetAutoFocus(false)
	fromfield:SetWidth(25)
	fromfield:SetHeight(24)
	
	--"To" field for moving items
	local tofield = CreateFrame('EditBox', "tofield", OptionsFrame, "InputBoxTemplate")
	tofield:SetPoint("TOP", MovingHeadingFont, "BOTTOM", 25, -3)
	tofield:SetAutoFocus(false)
	tofield:SetWidth(25)
	tofield:SetHeight(24)
	
	--"To" between the To and From fields
	local to = CreateFrame('Frame', "To", OptionsFrame)
	local toheadingfont = to:CreateFontString("to", "ARTWORK", "GameFontNormalSmallLeft")
	toheadingfont:SetText(L["to"])
	toheadingfont:SetPoint("TOP", MovingHeadingFont, "BOTTOM", -2, -9)
	
	--"Move Item" button
	local MoveButton = CreateFrame('Button', "MoveButton", OptionsFrame, "UIPanelButtonTemplate")
	MoveButton:SetPoint("TOP", toheadingfont, "BOTTOM", 0, -10)
	MoveButton:SetWidth(100)
    MoveButton:SetHeight(24)
	MoveButton:SetText(L["Move Item"])
	MoveButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:MoveItem(fromfield, tofield, UpgradeListMainFrame)
	end)
	return MovingHeading, fromfield, tofield, to, MoveButton
end

function UpgradeList:CreateDeleteSingleItem()
	--"Delete item" heading
	local DeletingHeading = CreateFrame('Frame', "DeletingHeading", OptionsFrame)
	local DeletingHeadingFont = DeletingHeading:CreateFontString("DeletingHeading", "ARTWORK", "GameFontNormalSmallLeft")
	DeletingHeadingFont:SetText(L["In the currently selected tab, delete item #:"])
	DeletingHeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP", -275, -90)
	DeletingHeadingFont:SetWidth(DeletingHeadingFont:GetStringWidth() + 5)
	DeletingHeadingFont:SetHeight(DeletingHeadingFont:GetStringHeight())
	
	--"Delete Item" button
	local DeleteButton = CreateFrame('Button', "DeleteButton", OptionsFrame, "UIPanelButtonTemplate")
	DeleteButton:SetPoint("TOP", DeletingHeadingFont, "BOTTOM", 0, -25)
	DeleteButton:SetWidth(100)
    DeleteButton:SetHeight(24)
	DeleteButton:SetText(L["Delete Item"])
	DeleteButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:DeleteItem(deletefield, UpgradeListMainFrame)
	end)
	
	--"Delete Item" field
	local deletefield = CreateFrame('EditBox', "deletefield", OptionsFrame, "InputBoxTemplate")
	deletefield:SetPoint("TOP", DeletingHeadingFont, "BOTTOM", 0, 0)
	deletefield:SetAutoFocus(false)
	deletefield:SetWidth(25)
	deletefield:SetHeight(24)
	
	return DeletingHeading, DeleteButton, deletefield
end

function UpgradeList:CreateDeleteManyItems()
	--MultiDelete heading text
	local MultiDeleteHeading = CreateFrame('Frame', "MultiDeleteHeading", OptionsFrame)
	local MultiDeleteHeadingFont = MultiDeleteHeading:CreateFontString("MultiDeleteHeading", "ARTWORK", "GameFontNormalSmallLeft")
	MultiDeleteHeadingFont:SetText(L["In the currently selected tab, delete item #'s:"])
	MultiDeleteHeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP", -275, -180)
	MultiDeleteHeadingFont:SetWidth(MultiDeleteHeadingFont:GetStringWidth() + 5)
	MultiDeleteHeadingFont:SetHeight(MultiDeleteHeadingFont:GetStringHeight())
	
	--"From" field for MultiDelete items
	local multifromfield = CreateFrame('EditBox', "multifromfield", OptionsFrame, "InputBoxTemplate")
	multifromfield:SetPoint("TOP", MultiDeleteHeadingFont, "BOTTOM", -38, -2)
	multifromfield:SetAutoFocus(false)
	multifromfield:SetWidth(25)
	multifromfield:SetHeight(24)
	
	--"To" field for MultiDelete items
	local multitofield = CreateFrame('EditBox', "multitofield", OptionsFrame, "InputBoxTemplate")
	multitofield:SetPoint("TOP", MultiDeleteHeadingFont, "BOTTOM", 38, -3)
	multitofield:SetAutoFocus(false)
	multitofield:SetWidth(25)
	multitofield:SetHeight(24)
	
	--"Through" between the To and From fields
	local multithrough = CreateFrame('Frame', "multiThrough", OptionsFrame)
	local multithroughheadingfont = multithrough:CreateFontString("multiThrough", "ARTWORK", "GameFontNormalSmallLeft")
	multithroughheadingfont:SetText(L["through"])
	multithroughheadingfont:SetPoint("TOP", MultiDeleteHeadingFont, "BOTTOM", -2, -9)
	
	--"Multi-Delete Item" button
	local MultiDeleteButton = CreateFrame('Button', "MultiDeleteButton", OptionsFrame, "UIPanelButtonTemplate")
	MultiDeleteButton:SetPoint("TOP", multithroughheadingfont, "BOTTOM", 0, -10)
	MultiDeleteButton:SetWidth(100)
    MultiDeleteButton:SetHeight(24)
	MultiDeleteButton:SetText(L["Delete Items"])
	MultiDeleteButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:DeleteSectionOfItems(multifromfield, multitofield, UpgradeListMainFrame)
	end)
	
	return MultiDeleteHeading, multifromfield, multitofield, multithrough, MultiDeleteButton
end

function UpgradeList:CreateStatusBar()
	local PaneBackdrop  = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	}
	
	statusbar = CreateFrame("Frame", "Status Bar", OptionsFrame)
	statusbar:SetPoint("BOTTOMLEFT", OptionsFrame, "BOTTOMLEFT", 15, 15)
	statusbar:SetPoint("BOTTOMRIGHT", OptionsFrame, "BOTTOMRIGHT", -15, 15)
	statusbar:SetHeight(24)
	statusbar:SetBackdrop(PaneBackdrop)
	statusbar:SetBackdropColor(0.1,0.1,0.1)
	statusbar:SetBackdropBorderColor(0.4,0.4,0.4)
	
	statusbarfont = statusbar:CreateFontString("statusbar", "ARTWORK", "GameFontWhite")
	statusbarfont:SetPoint("LEFT", statusbar, "LEFT", 7, 0)
	statusbarfont:SetHeight(22)
end

function UpgradeList:CreateItemTrackButton()
	local ItemTrackButton = CreateFrame('Button', "ItemTrackButton", OptionsFrame, "UIPanelButtonTemplate2")
	ItemTrackButton:SetPoint("TOPRIGHT", OptionsFrame, "TOPRIGHT", -50, -50)
	ItemTrackButton:SetWidth(175)
	ItemTrackButton:SetHeight(30)
	ItemTrackButton:SetText("Item Tracker")
	ItemTrackButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:DrawItemTrackWindow()
	end)
	return ItemTrackButton
end

function UpgradeList:CreatePointInputFrame()
	local PointInput = CreateFrame("Frame", "PointInput", UIParent)
	PointInput:SetFrameStrata("FULLSCREEN")
	PointInput:SetWidth(250)
	PointInput:SetHeight(200)
	
	PointInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
	edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	tile = true, tileSize = 32, edgeSize = 32, 
	insets = { left = 11, right = 11, top = 10, bottom = 10 }})

	PointInput:SetBackdropColor(0,0,0,1)
	PointInput:SetPoint("CENTER", UIParent, "CENTER")
	
	PointInput:Hide()
	
	local PointInputHeading = CreateFrame('Frame', "PointInputHeading", PointInput)
	PointInputFont = PointInputHeading:CreateFontString("PointInputHeading", "ARTWORK", "GameFontNormal")
	PointInputFont:SetText(L["Enter the number\n of points needed\n for this item:"])
	PointInputFont:SetJustifyH("CENTER")
	PointInputFont:SetPoint("TOP", PointInput, "TOP", 0, -15)
	PointInputFont:SetWidth(230)
	PointInputFont:SetHeight(PointInputFont:GetStringHeight())
	
	local PointInputBox = CreateFrame("Editbox", "PointInputBox", PointInput, "InputBoxTemplate")
	PointInputBox:SetPoint("CENTER", PointInput, "CENTER", 0, 0)
	PointInputBox:SetAutoFocus(true)
	PointInputBox:SetWidth(175)
	PointInputBox:SetHeight(27)
	PointInputBox:SetNumeric(1)
	PointInputBox:SetMaxLetters(5)
	
	local PointInputOK = CreateFrame('Button', "PointInputOK", PointInput, "UIPanelButtonTemplate")
	PointInputOK:SetPoint("BOTTOM", PointInput, "BOTTOM", -50, 20)
	PointInputOK:SetWidth(100)
	PointInputOK:SetHeight(24)
	PointInputOK:SetText(L["Ok"])
	PointInputOK:SetScript("OnClick", function (self, button, down)
		local pointnum = tonumber(PointInputBox:GetText())
		if pointnum == nil then
			UpgradeList:StatusBarUpdate(L["No number entered."], "red")
		elseif pointnum >= 0 then
			pointnum = math.floor(pointnum)
			UIDropDownMenu_SetText(ULLocationDropDown_Button, UIDropDownMenu_GetSelectedName(ULLocationDropDown_Button))
			PointInput:Hide()
			PointInputBox:SetText("")
			UpgradeListDBVar.char.SavedOptions.SelectedBoss = pointnum
			UpgradeListDBVar.char.SavedOptions.SelectedInstance = UIDropDownMenu_GetSelectedName(ULLocationDropDown_Button)
			UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = ""
			UpgradeList:EnableInput()
		else
			UpgradeList:StatusBarUpdate(L["Invalid number."], "red")
		end
	end)
	
	local PointInputCancel = CreateFrame('Button', "PointInputCancel", PointInput, "UIPanelButtonTemplate")
	PointInputCancel:SetPoint("LEFT", PointInputOK, "RIGHT", 0, 0)
	PointInputCancel:SetWidth(100)
	PointInputCancel:SetHeight(24)
	PointInputCancel:SetText(L["Cancel"])
	PointInputCancel:SetScript("OnClick", function (self, button, down)
		UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Location:"])
		PointInput:Hide()
		UpgradeList:EnableInput()
		PointInputBox:SetText("")
	end)
	
	return PointInput, PointInputHeading, PointInputBox, PointInputOK, PointInputCancel
end

function UpgradeList:CreateWeaponInputFrame()
	local WeaponInput = CreateFrame("Frame", "WeaponInput", UIParent)
	WeaponInput:SetFrameStrata("FULLSCREEN")
	WeaponInput:SetWidth(250)
	WeaponInput:SetHeight(200)
	
	WeaponInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }})

	WeaponInput:SetBackdropColor(0,0,0,1)
	WeaponInput:SetPoint("CENTER", UIParent, "CENTER")
	
	WeaponInput:Hide()
	
	local WeaponInputHeading = CreateFrame('Frame', "WeaponInputHeading", WeaponInput)
	local WeaponInputFont = WeaponInputHeading:CreateFontString("WeaponInputHeading", "ARTWORK", "GameFontNormal")
	WeaponInputFont:SetText(L["Select which list to add this item:"])
	WeaponInputFont:SetJustifyH("CENTER")
	WeaponInputFont:SetPoint("TOP", WeaponInput, "TOP", 0, -15)
	WeaponInputFont:SetWidth(230)
	WeaponInputFont:SetHeight(WeaponInputFont:GetStringHeight())
	
	--Main Hand weapon checkbox
	local MainHandCheck = CreateFrame('CheckButton', "MainHandCheck", WeaponInput, "UICheckButtonTemplate")
	MainHandCheck:SetPoint("CENTER", WeaponInput, "CENTER", -30, 25)
	MainHandCheck:SetChecked(true)
	_G[MainHandCheck:GetName() .. "Text"]:SetText(L["Main Hand"])
	MainHandCheck:SetScript("OnClick", function (self, button, down)
		OffHandCheck:SetChecked(false)
		MainHandCheck:SetChecked(true)
	end)
	
	--Off Hand weapon checkbox
	local OffHandCheck = CreateFrame('CheckButton', "OffHandCheck", WeaponInput, "UICheckButtonTemplate")
	OffHandCheck:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0)
	OffHandCheck:SetChecked(false)
	_G[OffHandCheck:GetName() .. "Text"]:SetText(L["Off Hand"])
	OffHandCheck:SetScript("OnClick", function (self, button, down)
		OffHandCheck:SetChecked(true)
		MainHandCheck:SetChecked(false)
	end)
	
	local WeaponInputOK = CreateFrame('Button', "WeaponInputOK", WeaponInput, "UIPanelButtonTemplate")
	WeaponInputOK:SetPoint("BOTTOM", WeaponInput, "BOTTOM", -50, 20)
	WeaponInputOK:SetWidth(100)
	WeaponInputOK:SetHeight(24)
	WeaponInputOK:SetText(L["Ok"])
	WeaponInputOK:SetScript("OnClick", function (self, button, down)
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(ItemEntryBox:GetText())
		local priorityfieldnum = tonumber(priorityfield:GetText())
		if priorityfieldnum ~= nil then
			priorityfieldnum = math.floor(priorityfieldnum)
		end
		if PrimarySpecRadioButton:GetChecked() then --Adding to Primary Spec List
			if MainHandCheck:GetChecked() then --If the user selects the Main Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.PrimarySpecList.Main.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.PrimarySpecList.Main.Row[UpgradeListDBVar.char.PrimarySpecList.Main.Count] = { UpgradeListDBVar.char.PrimarySpecList.Main.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() }
				elseif priorityfieldnum <= UpgradeListDBVar.char.PrimarySpecList.Main.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.PrimarySpecList.Main.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() })
					for i = UpgradeListDBVar.char.PrimarySpecList.Main.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][1] = i
					end
				end
				UpgradeListDBVar.char.PrimarySpecList.Main.Count = UpgradeListDBVar.char.PrimarySpecList.Main.Count + 1
				UpgradeList:StatusBarUpdate(L["Successfully added "] .. itemLink .. L[" to your Primary Spec's Main tab."], "green")
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13) and (UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil
				ItemEntryBox:SetText("")
				ItemEntryBox:ClearFocus()
				NoteEntryBox:SetText("")
				NoteEntryBox:ClearFocus()
				priorityfield:SetText("")
				priorityfield:ClearFocus()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, "Location:")
			elseif OffHandCheck:GetChecked() then --If the user selects the Off Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.PrimarySpecList.Off.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.PrimarySpecList.Off.Row[UpgradeListDBVar.char.PrimarySpecList.Off.Count] = { UpgradeListDBVar.char.PrimarySpecList.Off.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() }
				elseif priorityfieldnum <= UpgradeListDBVar.char.PrimarySpecList.Off.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.PrimarySpecList.Off.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() })
					for i = UpgradeListDBVar.char.PrimarySpecList.Off.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][1] = i
					end
				end
				UpgradeListDBVar.char.PrimarySpecList.Off.Count = UpgradeListDBVar.char.PrimarySpecList.Off.Count + 1
				UpgradeList:StatusBarUpdate(L["Successfully added "] .. itemLink .. L[" to your Primary Spec's Off tab."], "green")
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14) and (UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil
				ItemEntryBox:SetText("")
				ItemEntryBox:ClearFocus()
				NoteEntryBox:SetText("")
				NoteEntryBox:ClearFocus()
				priorityfield:SetText("")
				priorityfield:ClearFocus()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Location:"])
			else
				--Can't tell if the user selected main hand or off hand checkbox
			end
		else --Adding to Secondary Spec List
			if MainHandCheck:GetChecked() then --If the user selects the Main Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.SecondarySpecList.Main.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.SecondarySpecList.Main.Row[UpgradeListDBVar.char.SecondarySpecList.Main.Count] = { UpgradeListDBVar.char.SecondarySpecList.Main.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() }
				elseif priorityfieldnum <= UpgradeListDBVar.char.SecondarySpecList.Main.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.SecondarySpecList.Main.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() })
					for i = UpgradeListDBVar.char.SecondarySpecList.Main.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][1] = i
					end
				end
				UpgradeListDBVar.char.SecondarySpecList.Main.Count = UpgradeListDBVar.char.SecondarySpecList.Main.Count + 1
				UpgradeList:UpdateStatusBar(L["Successfully added "] .. itemLink .. L[" to your Secondary Spec's Main tab."], "green")
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13) and (UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil
				ItemEntryBox:SetText("")
				ItemEntryBox:ClearFocus()
				NoteEntryBox:SetText("")
				NoteEntryBox:ClearFocus()
				priorityfield:SetText("")
				priorityfield:ClearFocus()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, "Location:")
			elseif OffHandCheck:GetChecked() then --If the user selects the Off Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.SecondarySpecList.Off.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.SecondarySpecList.Off.Row[UpgradeListDBVar.char.SecondarySpecList.Off.Count] = { UpgradeListDBVar.char.SecondarySpecList.Off.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() }
				elseif priorityfieldnum <= UpgradeListDBVar.char.SecondarySpecList.Off.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.SecondarySpecList.Off.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() })
					for i = UpgradeListDBVar.char.SecondarySpecList.Off.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][1] = i
					end
				end
				UpgradeListDBVar.char.SecondarySpecList.Off.Count = UpgradeListDBVar.char.SecondarySpecList.Off.Count + 1
				UpgradeList:UpdateStatusBar(L["Successfully added "] .. itemLink .. L[" to your Secondary Spec's Off tab."], "green")
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14) and (UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil
				ItemEntryBox:SetText("")
				ItemEntryBox:ClearFocus()
				NoteEntryBox:SetText("")
				NoteEntryBox:ClearFocus()
				priorityfield:SetText("")
				priorityfield:ClearFocus()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Location:"])
			else
				UpgradeList:StatusBarUpdate(L["ERROR: Unable to determine if you selected Main Hand or Off Hand for this weapon"], "red")
			end
		end
		WeaponInput:Hide()
		OffHandCheck:SetChecked(false)
		MainHandCheck:SetChecked(true)
		UpgradeList:EnableInput()
	end)
	
	local WeaponInputCancel = CreateFrame('Button', "WeaponInputCancel", WeaponInput, "UIPanelButtonTemplate")
	WeaponInputCancel:SetPoint("LEFT", WeaponInputOK, "RIGHT", 0, 0)
	WeaponInputCancel:SetWidth(100)
	WeaponInputCancel:SetHeight(24)
	WeaponInputCancel:SetText(L["Cancel"])
	WeaponInputCancel:SetScript("OnClick", function (self, button, down)
		WeaponInput:Hide()
		OffHandCheck:SetChecked(false)
		MainHandCheck:SetChecked(true)
		UpgradeList:EnableInput()
	end)
	
	return WeaponInput, WeaponInputHeading, MainHandCheck, OffHandCheck, WeaponInputOK, WeaponInputCancel
end

function UpgradeList:CreateSpecInputFrame()
	local SpecInput = CreateFrame("Frame", "SpecInput", UIParent)
	SpecInput:SetFrameStrata("FULLSCREEN")
	SpecInput:SetWidth(250)
	SpecInput:SetHeight(200)
	
	SpecInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }})

	SpecInput:SetBackdropColor(0,0,0,1)
	SpecInput:SetPoint("CENTER", UIParent, "CENTER")
	
	SpecInput:Hide()
	
	local SpecInputHeading = CreateFrame('Frame', "SpecInputHeading", SpecInput)
	local SpecInputFont = SpecInputHeading:CreateFontString("SpecInputHeading", "ARTWORK", "GameFontNormal")
	SpecInputFont:SetText(L["Is your newly acquired item for your Main Spec or your Off Spec?"])
	SpecInputFont:SetJustifyH("CENTER")
	SpecInputFont:SetPoint("TOP", SpecInput, "TOP", 0, -15)
	SpecInputFont:SetWidth(230)
	SpecInputFont:SetHeight(SpecInputFont:GetStringHeight())
	
	--Main Spec selected
	local MainSpecCheck = CreateFrame('CheckButton', "MainSpecCheck", SpecInput, "UICheckButtonTemplate")
	MainSpecCheck:SetPoint("CENTER", SpecInput, "CENTER", -30, 25)
	MainSpecCheck:SetChecked(true)
	_G[MainSpecCheck:GetName() .. "Text"]:SetText(L["Main Spec"])
	MainSpecCheck:SetScript("OnClick", function (self, button, down)
		OffSpecCheck:SetChecked(false)
		MainSpecCheck:SetChecked(true)
	end)
	
	--Off Spec selected
	local OffSpecCheck = CreateFrame('CheckButton', "OffSpecCheck", SpecInput, "UICheckButtonTemplate")
	OffSpecCheck:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0)
	OffSpecCheck:SetChecked(false)
	_G[OffSpecCheck:GetName() .. "Text"]:SetText(L["Off Spec"])
	OffSpecCheck:SetScript("OnClick", function (self, button, down)
		OffSpecCheck:SetChecked(true)
		MainSpecCheck:SetChecked(false)
	end)
	
	local SpecInputOK = CreateFrame('Button', "SpecInputOK", SpecInput, "UIPanelButtonTemplate")
	SpecInputOK:SetPoint("BOTTOM", SpecInput, "BOTTOM", -50, 20)
	SpecInputOK:SetWidth(100)
	SpecInputOK:SetHeight(24)
	SpecInputOK:SetText(L["Ok"])
		
	local SpecInputCancel = CreateFrame('Button', "SpecInputCancel", SpecInput, "UIPanelButtonTemplate")
	SpecInputCancel:SetPoint("LEFT", SpecInputOK, "RIGHT", 0, 0)
	SpecInputCancel:SetWidth(100)
	SpecInputCancel:SetHeight(24)
	SpecInputCancel:SetText(L["Cancel"])
	SpecInputCancel:SetScript("OnClick", function (self, button, down)
		SpecInput:Hide()
		OffSpecCheck:SetChecked(false)
		MainSpecCheck:SetChecked(true)
		UpgradeList:EnableInput()
	end)
	
	return SpecInput, SpecInputHeading, MainSpecCheck, OffSpecCheck, SpecInputOK, SpecInputCancel
end

function UpgradeList:CreateMainHandorOffHandFrame()
	local MainHandorOffHand = CreateFrame("Frame", "MainHandorOffHand", UIParent)
	MainHandorOffHand:SetFrameStrata("FULLSCREEN")
	MainHandorOffHand:SetWidth(250)
	MainHandorOffHand:SetHeight(200)
	
	MainHandorOffHand:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }})

	MainHandorOffHand:SetBackdropColor(0,0,0,1)
	MainHandorOffHand:SetPoint("CENTER", UIParent, "CENTER")
	
	MainHandorOffHand:Hide()
	
	local MainHandorOffHandHeading = CreateFrame('Frame', "MainHandorOffHandHeading", MainHandorOffHand)
	local MainHandorOffHandFont = MainHandorOffHandHeading:CreateFontString("MainHandorOffHandHeading", "ARTWORK", "GameFontNormal")
	MainHandorOffHandFont:SetText(L["Is your newly acquired item for your Main Hand or your Off Hand?"])
	MainHandorOffHandFont:SetJustifyH("CENTER")
	MainHandorOffHandFont:SetPoint("TOP", MainHandorOffHand, "TOP", 0, -15)
	MainHandorOffHandFont:SetWidth(230)
	MainHandorOffHandFont:SetHeight(MainHandorOffHandFont:GetStringHeight())
	
	--Main Hand weapon check
	local WeaponInMain = CreateFrame('CheckButton', "WeaponInMain", MainHandorOffHand, "UICheckButtonTemplate")
	WeaponInMain:SetPoint("CENTER", MainHandorOffHand, "CENTER", -30, 25)
	WeaponInMain:SetChecked(true)
	_G[WeaponInMain:GetName() .. "Text"]:SetText(L["Main Hand"])
	WeaponInMain:SetScript("OnClick", function (self, button, down)
		WeaponInOff:SetChecked(false)
		WeaponInMain:SetChecked(true)
	end)
	
	--Off Hand weapon check
	local WeaponInOff = CreateFrame('CheckButton', "WeaponInOff", MainHandorOffHand, "UICheckButtonTemplate")
	WeaponInOff:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0)
	WeaponInOff:SetChecked(false)
	_G[WeaponInOff:GetName() .. "Text"]:SetText(L["Off Hand"])
	WeaponInOff:SetScript("OnClick", function (self, button, down)
		WeaponInOff:SetChecked(true)
		WeaponInMain:SetChecked(false)
	end)
	
	local MainHandorOffHandOK = CreateFrame('Button', "MainHandorOffHandOK", MainHandorOffHand, "UIPanelButtonTemplate")
	MainHandorOffHandOK:SetPoint("BOTTOM", MainHandorOffHand, "BOTTOM", -50, 20)
	MainHandorOffHandOK:SetWidth(100)
	MainHandorOffHandOK:SetHeight(24)
	MainHandorOffHandOK:SetText(L["Ok"])
	
	local MainHandorOffHandCancel = CreateFrame('Button', "MainHandorOffHandCancel", MainHandorOffHand, "UIPanelButtonTemplate")
	MainHandorOffHandCancel:SetPoint("LEFT", MainHandorOffHandOK, "RIGHT", 0, 0)
	MainHandorOffHandCancel:SetWidth(100)
	MainHandorOffHandCancel:SetHeight(24)
	MainHandorOffHandCancel:SetText(L["Cancel"])
	MainHandorOffHandCancel:SetScript("OnClick", function (self, button, down)
		MainHandorOffHand:Hide()
		WeaponInOff:SetChecked(false)
		WeaponInMain:SetChecked(true)
		UpgradeList:EnableInput()
	end)
	
	return MainHandorOffHand, MainHandorOffHandHeading, WeaponInMain, WeaponInOff, MainHandorOffHandOK, MainHandorOffHandCancel
end

function UpgradeList:CreateEditNoteFrame()
	local EditNote = CreateFrame("Frame", "EditNote", UIParent)
	EditNote:SetFrameStrata("FULLSCREEN")
	EditNote:SetWidth(360)
	EditNote:SetHeight(150)
	
	EditNote:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
	edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	tile = true, tileSize = 32, edgeSize = 32, 
	insets = { left = 11, right = 11, top = 10, bottom = 10 }})

	EditNote:SetBackdropColor(0,0,0,1)
	EditNote:SetPoint("CENTER", UIParent, "CENTER")
	
	EditNote:Hide()
	
	local EditNoteHeading = CreateFrame('Frame', "EditNoteHeading", EditNote)
	local EditNoteFont = EditNoteHeading:CreateFontString("EditNoteHeading", "ARTWORK", "GameFontNormal")
	EditNoteFont:SetText(L["Enter your new note:"])
	EditNoteFont:SetJustifyH("CENTER")
	EditNoteFont:SetPoint("TOP", EditNote, "TOP", 0, -15)
	EditNoteFont:SetWidth(230)
	EditNoteFont:SetHeight(EditNoteFont:GetStringHeight())
	
	local EditNoteBox = CreateFrame("Editbox", "EditNoteBox", EditNote, "InputBoxTemplate")
	EditNoteBox:SetPoint("CENTER", EditNote, "CENTER", 0, 0)
	EditNoteBox:SetAutoFocus(true)
	EditNoteBox:SetWidth(285)
	EditNoteBox:SetHeight(27)
	EditNoteBox:SetMaxLetters(28)
	
	local EditNoteOK = CreateFrame('Button', "EditNoteOK", EditNote, "UIPanelButtonTemplate")
	EditNoteOK:SetPoint("BOTTOM", EditNote, "BOTTOM", -50, 20)
	EditNoteOK:SetWidth(100)
	EditNoteOK:SetHeight(24)
	EditNoteOK:SetText(L["Ok"])
	
	local EditNoteCancel = CreateFrame('Button', "EditNoteCancel", EditNote, "UIPanelButtonTemplate")
	EditNoteCancel:SetPoint("LEFT", EditNoteOK, "RIGHT", 0, 0)
	EditNoteCancel:SetWidth(100)
	EditNoteCancel:SetHeight(24)
	EditNoteCancel:SetText(L["Cancel"])
	EditNoteCancel:SetScript("OnClick", function (self, button, down)
		EditNote:Hide()
		UpgradeList:EnableInput()
		EditNoteBox:SetText("")
	end)
	
	return EditNote, EditNoteHeading, EditNoteBox, EditNoteOK, EditNoteCancel
end

function UpgradeList:DrawItemTrackWindow()
	UpgradeListMainFrame:Hide()
	
	--Main Frame
	local TrackFrame = CreateFrame('Frame', "TrackFrame", UIParent)
	TrackFrame:SetFrameStrata("DIALOG")
	TrackFrame:SetWidth(500)
	TrackFrame:SetHeight(700)
	
	TrackFrame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }})
		
	TrackFrame:SetBackdropColor(0,0,0,1)
	TrackFrame:SetPoint("CENTER", UIParent, "CENTER")
	TrackFrame:SetMovable(true)
	TrackFrame:EnableMouse(true)
	TrackFrame:RegisterForDrag("LeftButton")
	TrackFrame:SetScript("OnDragStart", TrackFrame.StartMoving)
	TrackFrame:SetScript("OnDragStop", TrackFrame.StopMovingOrSizing)
	tinsert(UISpecialFrames, "TrackFrame")
	
	local TrackFrameTitle = TrackFrame:CreateTexture(nil, "ARTWORK")
	TrackFrameTitle:SetTexture("Interface/DialogFrame/UI-DialogBox-Header")
	TrackFrameTitle:SetWidth(300)
	TrackFrameTitle:SetHeight(64)
	TrackFrameTitle:SetPoint("TOP", TrackFrame, 0, 12)
	TrackFrame.texture = TrackFrameTitle
	
	local TrackFrameTitleText = TrackFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	TrackFrameTitleText:SetText('Item Tracker')
	TrackFrameTitleText:SetPoint("TOP", TrackFrame, 0, -3)
	
	--Close Button
	local CloseButton = CreateFrame('Button', "CloseButton", TrackFrame, "UIPanelCloseButton")
	CloseButton:SetPoint("TOPRIGHT", -1, -2)
	CloseButton:SetWidth(40)
	CloseButton:SetHeight(40)
	
	--[[
	local buttonTypes = {"Raid", "Dungeon", "Crafted", "Quest", "Reputation", "Purchasable", "World", "Other"}
	
	for i = 1, #buttonTypes, 1 do
		local name = buttonTypes[i] .. "CheckButton"
		print(name)
		[name] = CreateFrame('CheckButton', name, "UICheckButtonTemplate")
		[name]:SetHeight(30)
		[name]:SetWidth(30)
		[name]:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 50 * i, -50)
		_G([name]:GetName() .. "Text"):SetText(buttonTypes[i])
		[name]:SetScript("OnClick", function (self, button, down)
			if [name]:GetChecked() == 1 then
				[name]:SetChecked(true)
			else
				[name]:SetChecked(false)
			end
		end)
	end
	--]]
	--Check Buttons
	local RaidCheckButton = CreateFrame('CheckButton', "RaidCheckButton", TrackFrame, "UICheckButtonTemplate")
	RaidCheckButton:SetHeight(30)
	RaidCheckButton:SetWidth(30)
	RaidCheckButton:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 50, -50)
	_G[RaidCheckButton:GetName() .. "Text"]:SetText("Raid")
	RaidCheckButton:SetScript("OnClick", function (self, button, down)
		if RaidCheckButton:GetChecked() == 1 then
			RaidCheckButton:SetChecked(true)
		else
			RaidCheckButton:SetChecked(false)
		end
	end)
	
	local DungeonCheckButton = CreateFrame('CheckButton', "DungeonCheckButton", TrackFrame, "UICheckButtonTemplate")
	DungeonCheckButton:SetHeight(30)
	DungeonCheckButton:SetWidth(30)
	DungeonCheckButton:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 150, -50)
	_G[DungeonCheckButton:GetName() .. "Text"]:SetText("Dungeon")
	DungeonCheckButton:SetScript("OnClick", function (self, button, down)
		if DungeonCheckButton:GetChecked() == 1 then
			DungeonCheckButton:SetChecked(true)
		else
			DungeonCheckButton:SetChecked(false)
		end
	end)
	
	local CraftedCheckButton = CreateFrame('CheckButton', "CraftedCheckButton", TrackFrame, "UICheckButtonTemplate")
	CraftedCheckButton:SetHeight(30)
	CraftedCheckButton:SetWidth(30)
	CraftedCheckButton:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 250, -50)
	_G[CraftedCheckButton:GetName() .. "Text"]:SetText("Crafted")
	CraftedCheckButton:SetScript("OnClick", function (self, button, down)
		if CraftedCheckButton:GetChecked() == 1 then
			CraftedCheckButton:SetChecked(true)
		else
			CraftedCheckButton:SetChecked(false)
		end
	end)
	
	local QuestCheckButton = CreateFrame('CheckButton', "QuestCheckButton", TrackFrame, "UICheckButtonTemplate")
	QuestCheckButton:SetHeight(30)
	QuestCheckButton:SetWidth(30)
	QuestCheckButton:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 350, -50)
	_G[QuestCheckButton:GetName() .. "Text"]:SetText("Quest Reward")
	QuestCheckButton:SetScript("OnClick", function (self, button, down)
		if QuestCheckButton:GetChecked() == 1 then
			QuestCheckButton:SetChecked(true)
		else
			QuestCheckButton:SetChecked(false)
		end
	end)
	
	local ReputationCheckButton = CreateFrame('CheckButton', "ReputationCheckButton", TrackFrame, "UICheckButtonTemplate")
	ReputationCheckButton:SetHeight(30)
	ReputationCheckButton:SetWidth(30)
	ReputationCheckButton:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 50, -100)
	_G[ReputationCheckButton:GetName() .. "Text"]:SetText("Reputation\nReward")
	ReputationCheckButton:SetScript("OnClick", function (self, button, down)
		if ReputationCheckButton:GetChecked() == 1 then
			ReputationCheckButton:SetChecked(true)
		else
			ReputationCheckButton:SetChecked(false)
		end
	end)
	
	local PurchasableCheckButton = CreateFrame('CheckButton', "PurchasableCheckButton", TrackFrame, "UICheckButtonTemplate")
	PurchasableCheckButton:SetHeight(30)
	PurchasableCheckButton:SetWidth(30)
	PurchasableCheckButton:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 150, -100)
	_G[PurchasableCheckButton:GetName() .. "Text"]:SetText("Purchasable")
	PurchasableCheckButton:SetScript("OnClick", function (self, button, down)
		if PurchasableCheckButton:GetChecked() == 1 then
			PurchasableCheckButton:SetChecked(true)
		else
			PurchasableCheckButton:SetChecked(false)
		end
	end)
	
	local WorldCheckButton = CreateFrame('CheckButton', "WorldCheckButton", TrackFrame, "UICheckButtonTemplate")
	WorldCheckButton:SetHeight(30)
	WorldCheckButton:SetWidth(30)
	WorldCheckButton:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 250, -100)
	_G[WorldCheckButton:GetName() .. "Text"]:SetText("World Drop")
	WorldCheckButton:SetScript("OnClick", function (self, button, down)
		if WorldCheckButton:GetChecked() == 1 then
			WorldCheckButton:SetChecked(true)
		else
			WorldCheckButton:SetChecked(false)
		end
	end)
	
	local OtherCheckButton = CreateFrame('CheckButton', "OtherCheckButton", TrackFrame, "UICheckButtonTemplate")
	OtherCheckButton:SetHeight(30)
	OtherCheckButton:SetWidth(30)
	OtherCheckButton:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 350, -100)
	_G[OtherCheckButton:GetName() .. "Text"]:SetText("Other")
	OtherCheckButton:SetScript("OnClick", function (self, button, down)
		if OtherCheckButton:GetChecked() == 1 then
			OtherCheckButton:SetChecked(true)
		else
			OtherCheckButton:SetChecked(false)
		end
	end)	
	--[[
	local RaidDropdown = CreateFrame("Button", "RaidDropDown_Button", TrackFrame, "UIDropDownMenuTemplate")
	RaidDropDown_Button:SetPoint("TOPLEFT", TrackFrame, "TOPLEFT", 20, 450)
	RaidDropDown_Button:Show()
	
	function DropDown_InitializeDropDown(self, level)
		local info = UIDropDownMenu_CreateInfo()
		UpgradeList:RaidSubmenu(info)
	end
	
	function DropDown_DropDownOnLoad(self)
		UIDropDownMenu_Initialize(self, DropDown_InitializeDropDown)
	end
	
	function RaidDropDown_ButtonOnClick(self, button, down)
		local name = self:GetName()
		ToggleDropDownMenu(1, nil, RaidDropDown_DropDown, name, 0, 0)
	end
	
	UIDropDownMenu_Initialize(RaidDropDown_Button, DropDown_InitializeDropDown)
	UIDropDownMenu_SetWidth(RaidDropDown_Button, 150)
	UIDropDownMenu_SetButtonWidth(RaidDropDown_Button, 150)
	UIDropDownMenu_SetSelectedID(RaidDropDown_Button, 0)
	UIDropDownMenu_JustifyText(RaidDropDown_Button, "CENTER")
	UIDropDownMenu_SetText(RaidDropDown_Button, "test")
	--]]
	TrackFrame:Show()
end

function UpgradeList:DrawUI()
	UpgradeList:LoadDropdownItems()
	UpgradeList:LoadDungeonJournalInfo()
	
	--Create Scrolling Window
	UpgradeList:CreateScrollingWindow()

	--Create Main Frame (UpgradeListMainFrame)
	self.UI.UpgradeListMainFrame, self.UI.MF_titleBG, self.UI.MF_titleText, self.UI.VersionHeading, self.UI.TableContainer = UpgradeList:CreateMainFrame()
	
	--Create the UpgradeListDBVar.char.SavedOptions.tableUL by defining the columns, rows, height of the rows, then creating the UpgradeListDBVar.char.SavedOptions.tableUL
	UpgradeList:CreateScrollingItemFrame()
	
	--"Close" button
	self.UI.CloseButton = UpgradeList:CloseButton()
	
	--"ToggleOptions" button
	self.UI.ToggleButton = UpgradeList:ToggleOptionsButton()
	
	--"PrimarySpecView" button
	UpgradeList:PrimarySpecViewButton()
	
	--"SecondarySpecView" button
	UpgradeList:SecondarySpecViewButton()
	
	--Explanation of what the various clicks on the table do
	self.UI.ExplanationHeading = UpgradeList:ExplanationHeadingText()

	--Create all the 16 tabs for the equipment slots
	UpgradeList:TabCreate()
	
	--Create the options frame below the main frame
	self.UI.OptionsFrame = UpgradeList:CreateOptionsFrame()
	
	--Creation of the Item Entry Box
	self.UI.ItemEntryBox, self.UI.ItemEntryBoxHeading, self.UI.AutoItemIndicator, self.UI.AutoTexture = UpgradeList:CreateItemEntryBox()
	
	--Creation of the dropdown menus
	self.UI.DropDownHeading = UpgradeList:CreateDropdownMenu()
	
	--Creation of the Note Entry Box
	self.UI.NoteEntryBox, self.UI.NoteEntryBoxHeading = UpgradeList:CreateNoteEntryBox()
	
	--Primary & Secondary Spec Selector Radio Buttons
	self.UI.PrimarySpecRadioButton, self.UI.SecondarySpecRadioButton = UpgradeList:CreateSpecRadioButtons()

	--AddItemLocation
	self.UI.priortyfield, self.UI.PriorityBoxHeading = UpgradeList:AddItemLocation()	
	
	--"Add" button
	self.UI.AddButton = UpgradeList:CreateAddButton()

	--Moving an item stuff
	self.UI.MovingHeading, self.UI.fromfield, self.UI.tofield, self.UI.to, self.UI.MoveButton = UpgradeList:CreateMoveItem()
	
	--Deleting a single item stuff
	self.UI.DeletingHeading, self.UI.DeleteButton, self.UI.deletefield = UpgradeList:CreateDeleteSingleItem()
	
	--Deleting many items stuff
	self.UI.MultiDeleteHeading, self.UI.multifromfield, self.UI.multitofield, self.UI.multithrough, self.UI.MultiDeleteButton = UpgradeList:CreateDeleteManyItems()
	
	--Status Bar
	UpgradeList:CreateStatusBar()
	
	--Item Track Button
--	self.UI.ItemTrackButton = UpgradeList:CreateItemTrackButton()

	--Frame for entering the number of points needed for certain items
	self.UI.PointInput, self.UI.PointInputHeading, self.UI.PointInputBox, self.UI.PointInputOK, self.UI.PointInputCancel = UpgradeList:CreatePointInputFrame()
	
	--Frame for selecting whether a weapon is main hand or off hand.
	self.UI.WeaponInput, self.UI.WeaponInputHeading, self.UI.MainHandCheck, self.UI.OffHandCheck, self.UI.WeaponInputOK, self.UI.WeaponInputCancel = UpgradeList:CreateWeaponInputFrame()
	
	--Frame for asking whether an acquired upgrade is for Main Spec or Off Spec
	self.UI.SpecInput, self.UI.SpecInputHeading, self.UI.MainSpecCheck, self.UI.OffSpecCheck, self.UI.SpecInputOK, self.UI.SpecInputCancel = UpgradeList:CreateSpecInputFrame()
	
	--Frame for asking whether an acquired upgrade is for Main Hand or Off Hand
	self.UI.MainHandorOffHand, self.UI.MainHandorOffHandHeading, self.UI.WeaponInMain, self.UI.WeaponInOff, self.UI.MainHandorOffHandOK, self.UI.MainHandorOffHandCancel = UpgradeList:CreateMainHandorOffHandFrame()

	--Frame for entering a new note
	self.UI.EditNote, self.UI.EditNoteHeading, self.UI.EditNoteBox, self.UI.EditNoteOK, self.UI.EditNoteCancel = UpgradeList:CreateEditNoteFrame()
	
	UpgradeListMainFrame:Hide()
end